@@ -1,28 +1,30 @@
 #' PCA for tall matrix
-#' 
+#'
 #' Efficient PCA for a tall matrix (many more rows than columns). Uses the SVD
 #'  of the covariance matrix. The dimensionality of the result can be preset
-#'  with \code{Q} or estimated with PESEL. 
-#' 
-#' @param X The tall numeric matrix for which to compute the PCA. For fMRI data, 
+#'  with \code{Q} or estimated with PESEL.
+#'
+#' @param X The tall numeric matrix for which to compute the PCA. For fMRI data,
 #'  \code{X} should be \code{V} brain locations by \code{T} timepoints.
 #' @param center Center the columns of \code{X}? Default: \code{TRUE}. Set to
-#'  \code{FALSE} if already centered. Centered data is required to compute PCA. 
-#' @param Q Number of latent dimensions to estimate. If \code{NULL} (default), 
+#'  \code{FALSE} if already centered. Centered data is required to compute PCA.
+#' @param Q Number of latent dimensions to estimate. If \code{NULL} (default),
 #'  estimated using PESEL (Sobczyka et al. 2020).
-#' @param Q_max Maximal number of principal components for automatic 
+#' @param Q_max Maximal number of principal components for automatic
 #'  dimensionality selection with PESEL. Default: \code{100}.
 #' @param Vdim Number of principal directions to obtain. Default: \code{0}. Can
 #'  also be \code{"Q"} to set equal to the value of \code{Q}. Note that setting
 #'  this value less than \code{Q} does not speed up computation time, but does
 #'  save on memory. Note that the directions will be with respect to \code{X},
 #'  not its covariance matrix.
-#' 
+#'
 #' @export
-#' 
+#'
+#' @importFrom stats cor 
+#'
 #' @return The SVD decomposition
-#' 
-#' @examples 
+#'
+#' @examples
 #' U <- matrix(rnorm(900), nrow=300, ncol=3)
 #' V <- matrix(rnorm(15), nrow=3, ncol=5)
 #' PCA(U %*% V)
@@ -44,9 +46,9 @@ PCA <- function(X, center=TRUE, Q=NULL, Q_max=100, Vdim=0) {
     TOL <- 1e-8
     if (max(abs(colMeans(X))) > TOL) stop('Columns of X must be centered')
   }
-  
+
   # Determine PCA dimensionality.
-  if(is.null(Q)){
+  if (is.null(Q)) {
     if (!requireNamespace("pesel", quietly = TRUE)) {
       stop("Package \"pesel\" needed to read input data. Please install it", call. = FALSE)
     }
@@ -56,7 +58,43 @@ PCA <- function(X, center=TRUE, Q=NULL, Q_max=100, Vdim=0) {
   if (Vdim > Q) { warning("Vdim > Q, so setting Vdim to Q."); Vdim <- Q }
 
   # Perform dimension reduction.
-  out <- svd(crossprod(X) / (nV-1), nu=Q, nv=0)
+  out <- tryCatch(
+    { svd(crossprod(X) / (nV-1), nu=Q, nv=0) },
+    error = function(cond) {
+      message(cond)
+      z <- crossprod(X) / (nV-1)
+
+      # If `svd` fails:
+      # First, try detecting and dropping co-linear columns, and running `svd` again.
+      zcor <- stats::cor(z)
+      zcor[upper.tri(zcor)] <- 0
+      diag(zcor) <- 0
+      zdrop <- apply(zcor, 2, function(cc) any(abs(cc) > .99))
+      if (any(zdrop)) {
+        if (sum(!zdrop) < Q) {
+          stop(cat(sum(zdrop), " co-linear columns detected, but dropping them would result in less than Q columns remaining."))
+        }
+        cat("Dropping", sum(zdrop), "co-linear columns.\n")
+        z <- z[,!zdrop]
+        z <- svd(z, nu=Q, nv=0)
+      } else {
+        # If no co-linear columns detected, try `corpcor:fast.svd`.
+        if (!requireNamespace("corpcor", quietly = TRUE)) {
+          stop(
+            "`svd` failed, and the backup routine `corpcor::fast.svd` ",
+            "is not available since the Package \"corpcor\" is needed. ",
+            "Please install it.", call. = FALSE
+          )
+        }
+        cat("Trying `corpcor::fast.svd`.\n")
+        z <- corpcor::fast.svd(z)[c("u", "d", "v")]
+        names(z) <- toupper(names(z))
+        z$U <- z$U[,seq(Q)]
+        z$V <- NULL
+      }
+      z
+    }
+  )
 
   # Compute directions. (out$v would have the directions for XtX, not X.)
   if (Vdim > 0) {
