@@ -52,20 +52,22 @@ scale_med <- function(mat, TOL=1e-8, drop_const=TRUE, doRows=FALSE){
 
 #' Scale the BOLD timeseries
 #'
-#' @param BOLD Input fMRI data (V x T)
-#' @param scale Option for scaling units.
-#'
-#' 	If \code{"auto"} (default), will use mean scaling except if demeaned data
-#' 	is detected, in which case sd scaling will be used instead.
-#'
+#' @param BOLD fMRI data as a locations by time (\eqn{V \times T}) numeric 
+#' 	matrix.
+#' @param scale Option for scaling the BOLD response.
+#' 
+#' 	\code{"auto"} (default) will use \code{"mean"} scaling except if demeaned 
+#'  data is detected (if any mean is less than one), in which case \code{"sd"}
+#'  scaling will be used instead.
+#' 
 #' 	\code{"mean"} scaling will scale the data to percent local signal change.
-#'
+#' 
 #' 	\code{"sd"} scaling will scale the data by local standard deviation.
-#'
-#' 	\code{"none"} will only center the data, not scale it.
-#' @param transpose Check orientation of data, which, if TRUE, will transpose
-#' 	the data when the number of time points is greater than the number of voxels.
-#' 	Note: this is not always true for subcortical regions.
+#' 
+#' 	\code{"none"} will only center the data, not scale it. 
+#' @param transpose Transpose \code{BOLD} if there are more columns than rows?
+#' 	(Because we usually expect the number of voxels to exceed the number of time
+#' 	points.) Default: \code{TRUE}.
 #'
 #' @return Scale to units of percent local signal change and centers
 #'
@@ -74,19 +76,18 @@ scale_med <- function(mat, TOL=1e-8, drop_const=TRUE, doRows=FALSE){
 scale_timeseries <- function(BOLD, scale=c("auto", "mean", "sd", "none"), transpose = TRUE){
 
 	BOLD <- as.matrix(BOLD)
-	nvox <- nrow(BOLD)
-	ntime <- ncol(BOLD)
-
 	scale <- match.arg(scale, c("auto", "mean", "sd", "none"))
+	stopifnot(is_1(transpose, "logical"))
 
 	# Check orientation, send warning message and transpose if necessary.
-	if((ntime > nvox) & transpose){
-		warning('More columns than rows. Transposing matrix so rows are data locations and columns are time points')
+	if ((ncol(BOLD) > nrow(BOLD)) & transpose) {
+		warning('More columns than rows. Transposing matrix so rows are data locations and columns are time points.')
 		BOLD <- t(BOLD)
-		nvox <- nrow(BOLD)
-		ntime <- ncol(BOLD)
 	}
 
+	nvox <- nrow(BOLD)
+	ntime <- ncol(BOLD)
+
 	# Get `v_means`, the mean over time for each location (the mean image)
 	v_means <- rowMeans(BOLD, na.rm=TRUE)
 	v_means_min <- min(v_means, na.rm = TRUE)
@@ -101,20 +102,18 @@ scale_timeseries <- function(BOLD, scale=c("auto", "mean", "sd", "none"), transp
 	BOLD <- BOLD - v_means
 	if (scale == "mean") {
 		if (v_means_min < .1) {
-			stop("Some local means are less than 0.1. Please set `scale_BOLD = 'none'` or set `meanTol` > .1 in the call to BayesGLM.")
+			stop("Some local means are less than 0.1. Please set `scale_BOLD` to `'none'` or `'sd'`.")
 		} else if (v_means_min < 1) {
 			warning("Scaling to percent signal change when locations have means less than 1 may cause errors or produce aberrant results.")
 		}
-		BOLD <- t(100*BOLD / v_means)
+		BOLD <- 100*BOLD / v_means
 	} else if (scale == "sd") {
 		v_sd <- sqrt(apply(BOLD, 1, var, na.rm=TRUE))
 		v_sd[is.na(v_sd)] <- 0
 		if (min(v_sd) < 1e-6) {
-			stop("Some local sds are less than 1e-6. Please set `scale_BOLD = 'none' or set `varTol` > 1e-3 in the call to BayesGLM.")
+			stop("Some local sds are less than 1e-6. Please set `scale_BOLD` to `'none'`.")
 		}
-		BOLD <- t(BOLD / v_sd)
-	} else {
-		BOLD <- t(BOLD)
+		BOLD <- BOLD / v_sd
 	}
 
 	BOLD
